unit wp14u1;

(*A program to assemble a page of HTML to display
  graphics, as a photo album.

  The program is "driven" by a script held in a simple
  text file, called the source file.

  The graphics files to be displayed are assumed to
  reside in the same folder as the source file. The
  output file is also created in that folder.

  The program is currently written assuming that the
  image files with be jpg format.
  =====

  The program is also meant to be an example of ways
  of doing things with Delphi. If you want to learn
  more about Delphi, visit....

  http://ourworld.compuserve.com/homepages/TK_Boyd/Tut.htm

  ... or, if you can't find that, use Google or similar
  search for "Sheepdog Software" / "Delphi Tutorials"
  =====

  The HTML generated is in three parts:

  First section... <html>, etc. It is possible
    to specify some introductory material via
    the script file.

  Middle section... Multiple IDBs: Image Display Blocks
    Each IDB specifies everything to display one image,
    its description, navigation buttons, horizontal
    divider, etc.

  Ending section... Again, some material can be
    specified via the script file.
  =====

  Script file format....

  The script file is composed of the following.

  Items in {} are descriptions of things, e.g.
  FRED{number} would specify something like FRED2
  or FRED 428, etc.

  Items marked {O} are Optional

  Note: most commands begin with a '!'

  MATERIAL IN CAPS FROM HERE: DESCRIPTION OF SCRIPT
  FILE, NOT TO BE INCLUDED IN IT...

  FIRST LINE MUST BE: Data for WP14
  SECOND: This file ver:{version ID for script}

  THEN FOLLOW NONE OR MORE OF THE FOLLOWING....

  {O}!op:{name for output file. Overrides interactive spec} TO BE IMPLEMENTED STILL
  {O}!sr:{screen resolution, horizontal, typically 640 or 800}TO BE IMPLEMENTED STILL
  {O}!sp:{Serial Prefix: Characters to go in front of photo indexes seen by user of the HTML}TO BE IMPLEMENTED STILL

  {O}!SH {nothing more on this line... marks start of
       multiple lines to be inserted in the html as
       header section for output, e.g. introduction,
       links to other pages.
     !EH {end of header section}

  AFTER THE ABOVE, ONE OR MORE IDB SPECS...
       Other display elements are hard coded- e.g. a <hr> at the start
       of the IDB
  !=
  {Image file name, without ".jpg", eg. pic1 if pic1.jpg exists}
  {Either w{number} or h{number} w100 makes image occupy the full
     width of the screen. h50 makes the height of the image 50%
     of the screen width (as specified with !SR, or a default value
     if !SR was not incorporated in the script file. MUST have one or the
     other. Use h0 or w0 to have image display at whatever size it was
     stored. NB: Browser can cope with having just width or height
     specified. For this application, should not have both.
  {O}{Then none or more lines of html. They will be enclosed in <p> and </p>
  !SP{number} WILL INSERT number <BR>s, for a vertical gap. (NOT optional)
  {O}{blank line(s) between IDB specs}

FOLLOWING THE IDBs

  {O}!sf TO START A BLOCK OF FOOTER TO GO INTO THE HTML
     !ef REQUIRED IF !sf used

AND FINALLY...
  !ESF

(... meaning End of Script File)


  *)

(*Started 17 November 01, 7NC, to compile
  albums of Galap/MP trip.*)

(*The term 'folder' is used throughout for what was once called a directory*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

const ver='17 Nov 01';
   (*Written with Delphi 2*)
   DefaultiScrRes=640;(*See decl of iScrRes*)

type
  Twp14f1 = class(TForm)
    buQuit: TButton;
    laSource: TLabel;
    Label1: TLabel;
    buDoIt: TButton;
    eSrc: TEdit;
    eOP: TEdit;
    buSrc: TButton;
    buOP: TButton;
    laWrkFld: TLabel;
    eWrkFld: TEdit;
    buWrkFld: TButton;
    cbOverwrite: TCheckBox;
    meSrc: TMemo;
    procedure buQuitClick(Sender: TObject);
    procedure buWrkFldClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure eSrcChange(Sender: TObject);
    procedure cbOverwriteClick(Sender: TObject);
    procedure eWrkFldExit(Sender: TObject);
    procedure eSrcExit(Sender: TObject);
    procedure eOPExit(Sender: TObject);
    procedure buDoItClick(Sender: TObject);
  private
    { Private declarations }
    sTmp,sTmp2,sTmp3:string;
    dfOut:textfile;
    iTmp,iScrRes:integer;
      (*iScrRes holds the width of screen (in pixels) the HTML is
        designed for. For each image, a width or height can be
        specified. It is specified as a % of iScrRes. Therefore,
        simply changing the iScrRes spec will re-size all of
        the images in a new HTML page. Typically 640 or 800.*)
    procedure ManageOverwrite;
  public
    { Public declarations }
  end;

var
  wp14f1: Twp14f1;

implementation

{$R *.DFM}
{$R+} (*Turn on range checking. Nothing to do with previous*)

procedure Twp14f1.ManageOverwrite;
(*This checks whether the specified output file
already exists. If it does, disable the "Do It"
button. The button may be re-enabled either by
specifying an output file which does not exist,
or by ticking the "Overwrite existing" checkbox.

Interaction with disabling of buDoIt by code
checking eWrkFld.text and eSrc.text are valid
is incomplete and finicky. Beware!
*)
begin
cbOverwrite.checked:=false;
if fileexists(eWrkFld.text+eOP.text) then begin
buDoIt.enabled:=false;
end (*no ; here*)
else begin
buDoIt.enabled:=true;
end(*else*)
end;


procedure Twp14f1.buQuitClick(Sender: TObject);
begin
application.terminate;
end;

procedure Twp14f1.buWrkFldClick(Sender: TObject);
begin
showmessage('Not written yet, but values in edit box may be changed.');
end;

procedure Twp14f1.FormCreate(Sender: TObject);
begin
{(**********
assignfile(dfOut,'C:\wp14\tmp.txt');
rewrite(dfOut);(*Will overwrite existing file of same name if present*)
writeln(dfOut,'Hi');
closefile(dfOut);
application.terminate;
*********)}
ManageOverWrite;
if not fileexists(eWrkFld.text+eSrc.text) then buDoIt.enabled:=false;
end;

procedure Twp14f1.eSrcChange(Sender: TObject);
(*Handle possibility that a non-existant source has been
  specified.*)
begin
ManageOverwrite;
end;

procedure Twp14f1.cbOverwriteClick(Sender: TObject);
(*Ensure that overwriting of output file is possible, but
  only upon explicit election by user.*)
begin
if cbOverwrite.checked then buDoIt.enabled:=true (*no ; here*)
   else buDoIt.enabled:=false;
end;

procedure Twp14f1.eWrkFldExit(Sender: TObject);
(*Ensure folderspec does end with '\'*)
begin
sTmp:=eWrkFld.text;
if copy(sTmp,length(sTmp),1)<>'\' then eWrkFld.text:=eWrkFld.text+'\';
end;

procedure Twp14f1.eSrcExit(Sender: TObject);
(*Ensure filespec does NOT start with '\'*)
begin
if eSrc.text[1]='\' then eSrc.text:=copy(eSrc.text,2,length(eSrc.text)-1);
end;

procedure Twp14f1.eOPExit(Sender: TObject);
(*Ensure filespec does NOT start with '\'*)
begin
if eOP.text[1]='\' then eOP.text:=copy(eOP.text,2,length(eSrc.text)-1);
end;

procedure Twp14f1.buDoItClick(Sender: TObject);
(*Create output file according to instructions in script file*)
(* see ------> for start of buDoItClick's main block*)
var boOkSoFar:boolean;
    iNextLineToProc,iOneAfterLast:integer;

function RemoveSpaces(sTmp:string):string;
(*Strips any leading spaces*)
begin
repeat
if length(sTmp)>0 then
  if sTmp[1]=' ' then sTmp:=copy(sTmp,2,length(sTmp)-1);
until (length(sTmp)=0) or (sTmp[1]<>' ');
result:=sTmp;
end;

procedure CloseOPFile;
(*Made a procedure because it could be needed in
  several places*)
begin
closefile(dfOut);
end;

procedure FetchSrc;
(*Transfer contents of source (script) file to hidded memo
called meSrc (for easy access).
Do preliminary vetting of data format.*)

begin (*FetchScr*)
(*do more prelim vett*)
meSrc.lines.loadfromfile(eWrkFld.text+eSrc.text);
(*First line is index 0...*)
if uppercase(meSrc.lines[0])<>'DATA FOR WP14' then begin
     showmessage('Source file must begin with "Data for WP14"');
     boOKSoFar:=false;
     end;
(*Set up some variables with indexes to lines of meSrc, after
  removing any blank or spaces-only lines at end of file.*)
iNextLineToProc:=1;
iOneAfterLast:=meSrc.lines.count+1;
while (RemoveSpaces(meSrc.lines[iOneAfterLast-1])='')
   and (iOneAfterLast>1)
     do begin
       meSrc.lines.delete(iOneAfterLast-1);
       dec(iOneAfterLast);
       end;
if uppercase(meSrc.lines[iOneAfterLast-1])<>'!ESF' then begin
     showmessage('Last line of file must be "!ESF" (<command> End Script File)');
     boOKSoFar:=false;
     end;

iScrRes:=DefaultiScrRes;

end;

procedure OpenOP;
(*Set up a filehandle, dfOut, for the output file, leave open.*)
(*Should this have more error handling?*)
begin
assignfile(dfOut,eWrkFld.text+eOP.text);
rewrite(dfOut);(*Will overwrite existing file of same name if present*)
(*if no joy then boOKSoFar:=false;*)
end;

procedure CreateFirstPart;
(*Write first part of html to output file.
First part covers everything down to first IDB.
Leave file open unless fatal error encountered.*)
begin
(*To be written... code to recognise and act on First Part commands...*)
Writeln(dfOut,'<html>');
Writeln(dfOut,'<head><title>WP14 Generated File</title>');
Writeln(dfOut,'</head>');
Writeln(dfOut,'');
Writeln(dfOut,'<body bgcolor="#ffffff">');
Writeln(dfOut,'<center><h2>WP14 Generated File</h2></center>');
Writeln(dfOut,'');
Writeln(dfOut,'<!-- end of beginning... Image Display Blocks (IDBs) begin -->');
Writeln(dfOut,'');
Writeln(dfOut,'');

while (meSrc.lines[iNextLineToProc]<>'!=')
  and (iNextLineToProc<iOneAfterLast)
    do inc(iNextLineToProc);

{writeln(dfOut,meSrc.lines[iNextLineToProc]+'debug');}

if iNextLineToProc=iOneAfterLast then begin
      showmessage('Didn''t find any != commands marking Image Display Block spec');
      CloseOPFile;
      boOKSoFar:=false;
      end;
end;

procedure CreateImageRefs;
(*Write IDBs to output file.
On entry: iNextLineToProc should point to != line
On exit: iNextLineToProc should point to first line after last IDB spec
NB: Success of logic of this is highly dependent on keeping to plan for
  format of IDBs!
Should be:
!=
{filename}
{dimension... NOT optional... use 0 to flag "Display full size"}
{O}comment line(s)
!SF{number}... NOT optional

Leave file open unless fatal error encountered.*)
begin
repeat
{incomplete...}
Writeln(dfOut,'<hr>');
Writeln(dfOut,'<br><br>');
inc(iNextLineToProc);
sTmp:=meSrc.lines[iNextLineToProc];(*Pick up file's name*)
sTmp:=sTmp+'.jpg';

inc(iNextLineToProc);
sTmp2:=uppercase(meSrc.lines[iNextLineToProc]);(*Pick up dimensioning info*)
(*... and deal with it...*)
sTmp3:='error';
if sTmp2[1]='W' then sTmp3:=' width=';
if sTmp2[1]='H' then sTmp3:=' height=';
{if sTmp3='error'... provide handling...}
iTmp:=strtoint(copy(sTmp2,2,length(sTmp2)-1));
(*Provide error handling for times sTmp2 has bad data?*)
iTmp:=trunc(iTmp*iScrRes/100);
if iTmp=0 then sTmp3:='' (*no ; here*)
  else sTmp3:=sTmp3+inttostr(itmp);

if not fileexists(eWrkFld.text+sTmp) then
  showmessage('Script asks for '+eWrkFld.text+sTmp+'.jpg'+' but it wasn''t found. '+
    'HTML will be assembled with ref to it anyway.');

Writeln(dfOut,'<img src="'+sTmp+'"'+sTmp3+'>');
Writeln(dfOut,'<p>');
repeat
inc(iNextLineToProc);
sTmp:=meSrc.lines[iNextLineToProc];(*Pick up possible comment, or !SP{number}*)
if uppercase(copy(sTmp,1,3))<>'!SP' then
  Writeln(dfOut,sTmp);
until uppercase(copy(sTmp,1,3))='!SP';
inc(iNextLineToProc);(*Move pointer forward, in preparation for
     processing next command, but first, deal with !SP{number}*)
Writeln(dfOut,'</p>');
{add deal with !SPnumber code here}

(*Skip any blank lines before next !=*)
while RemoveSpaces(meSrc.lines[iNextLineToProc])='' do inc(iNextLineToProc);
(*No need to test for eof over-run.. already guaranteed at least an !ESF*)


Writeln(dfOut,'<br><br>');
Writeln(dfOut,'');
until meSrc.lines[iNextLineToProc]<>'!=';
(*No need to test for eof over-run.. already guaranteed at least an !ESF*)

Writeln(dfOut,'');
Writeln(dfOut,'<!-- end of IDBs... final section begins -->');
Writeln(dfOut,'');
Writeln(dfOut,'');


(*if no joy then CloseOPFile;boOKSoFar:=false;*)
end;

procedure CreateFinalPart;
(*Write everything to follow last IDB to output file.
Leave file open unless fatal error encountered.*)
{bits to finish here....}
begin
Writeln(dfOut,'</body>');
Writeln(dfOut,'</html>');
(*if no joy then CloseOPFile;boOKSoFar:=false;*)
end;

(* ---------------------------------------->  *)
begin       (*Main block of buDoIt*)
(*I wrote the 9 lines following this very early in
  the project, AFTER thinking carefully about how
  the program would behave. If the thinking was done
  right, then the following skeleton should work
  well, and breaking it up like this makes program
  development go well. If you find youself having
  to alter the underlying skeleton as you go along,
  you will have bad experiences! PLAN FIRST!!*)

if fileexists(eWrkFld.text+eSrc.text) then begin
   boOKSoFar:=true;
   FetchSrc;
   if boOkSoFar then OpenOP;
   if boOkSoFar then CreateFirstPart;
   if boOkSoFar then CreateImageRefs;
   if boOkSoFar then CreateFinalPart;
   if boOkSoFar then CloseOPFile;
end (*no ; here*)
else begin
buDoIt.enabled:=false;
showmessage('Source file and/or working foldernot valid.');
(*Better input filtering can be implemented...
disable buDoIt AND cbOverwrite, and give guidance, whenever
source/folder invalid.
Left til later 'cause disentangling buDoIt enables from
handling of OP file existance tedious.*)
end(*else*)
end;(*buDoItClick*)

end.
